"""
Define a category on NSObject with some useful methods.
"""
import sys

import objc

if sys.version_info[0] == 2:

    def _str(v):
        return v

    exec(
        """\
def _raise(exc_type, exc_value, exc_trace):
    raise exc_type, exc_value, exc_trace
"""
    )
else:

    def _str(v):
        if isinstance(v, str):
            return v
        return v.decode("ascii")

    def _raise(exc_type, exc_value, exc_trace):
        raise exc_type(exc_value).with_traceback(exc_trace)


NSObject = objc.lookUpClass("NSObject")


class NSObject(objc.Category(NSObject)):
    @objc.namedSelector(b"_pyobjc_performOnThread:")
    def _pyobjc_performOnThread_(self, callinfo):
        try:
            sel, arg = callinfo
            m = getattr(self, _str(sel))
            m(arg)
        except:  # noqa: E722, B001
            import traceback

            traceback.print_exc(file=sys.stderr)

    @objc.namedSelector(b"_pyobjc_performOnThreadWithResult:")
    def _pyobjc_performOnThreadWithResult_(self, callinfo):
        try:
            sel, arg, result = callinfo
            m = getattr(self, _str(sel))
            r = m(arg)
            result.append((True, r))
        except:  # noqa: E722, B001
            result.append((False, sys.exc_info()))

    if hasattr(NSObject, "performSelector_onThread_withObject_waitUntilDone_"):

        @objc.namedSelector(
            b"pyobjc_performSelector:onThread:withObject:waitUntilDone:"
        )
        def pyobjc_performSelector_onThread_withObject_waitUntilDone_(
            self, aSelector, thread, arg, wait
        ):
            """
            A version of performSelector:onThread:withObject:waitUntilDone: that
            will log exceptions in the called method (instead of aborting the
            NSRunLoop on the other thread).
            """
            self.performSelector_onThread_withObject_waitUntilDone_(
                b"_pyobjc_performOnThread:", thread, (aSelector, arg), wait
            )

        @objc.namedSelector(
            b"pyobjc_performSelector:onThread:withObject:waitUntilDone:modes:"
        )
        def pyobjc_performSelector_onThread_withObject_waitUntilDone_modes_(
            self, aSelector, thread, arg, wait, modes
        ):
            """
            A version of performSelector:onThread:withObject:waitUntilDone:modes:
            that will log exceptions in the called method (instead of aborting the
            NSRunLoop on the other thread).
            """
            self.performSelector_onThread_withObject_waitUntilDone_modes_(
                b"_pyobjc_performOnThread:", thread, (aSelector, arg), wait, modes
            )

    @objc.namedSelector(b"pyobjc_performSelector:withObject:afterDelay:")
    def pyobjc_performSelector_withObject_afterDelay_(self, aSelector, arg, delay):
        """
        A version of performSelector:withObject:afterDelay:
        that will log exceptions in the called method (instead of aborting the
        NSRunLoop).
        """
        self.performSelector_withObject_afterDelay_(
            b"_pyobjc_performOnThread:", (aSelector, arg), delay
        )

    @objc.namedSelector(b"pyobjc_performSelector:withObject:afterDelay:inModes:")
    def pyobjc_performSelector_withObject_afterDelay_inModes_(
        self, aSelector, arg, delay, modes
    ):
        """
        A version of performSelector:withObject:afterDelay:inModes:
        that will log exceptions in the called method (instead of aborting the
        NSRunLoop).
        """
        self.performSelector_withObject_afterDelay_inModes_(
            b"_pyobjc_performOnThread:", (aSelector, arg), delay, modes
        )

    if hasattr(NSObject, "performSelectorInBackground_withObject_"):

        @objc.namedSelector(b"pyobjc_performSelectorInBackground:withObject:")
        def pyobjc_performSelectorInBackground_withObject_(self, aSelector, arg):
            """
            A version of performSelectorInBackground:withObject:
            that will log exceptions in the called method (instead of aborting the
            NSRunLoop).
            """
            self.performSelectorInBackground_withObject_(
                b"_pyobjc_performOnThread:", (aSelector, arg)
            )

    @objc.namedSelector(b"pyobjc_performSelectorOnMainThread:withObject:waitUntilDone:")
    def pyobjc_performSelectorOnMainThread_withObject_waitUntilDone_(
        self, aSelector, arg, wait
    ):
        """
        A version of performSelectorOnMainThread:withObject:waitUntilDone:
        that will log exceptions in the called method (instead of aborting the
        NSRunLoop in the main thread).
        """
        self.performSelectorOnMainThread_withObject_waitUntilDone_(
            b"_pyobjc_performOnThread:", (aSelector, arg), wait
        )

    @objc.namedSelector(
        b"pyobjc_performSelectorOnMainThread:withObject:waitUntilDone:modes:"
    )
    def pyobjc_performSelectorOnMainThread_withObject_waitUntilDone_modes_(
        self, aSelector, arg, wait, modes
    ):
        """
        A version of performSelectorOnMainThread:withObject:waitUntilDone:modes:
        that will log exceptions in the called method (instead of aborting the
        NSRunLoop in the main thread).
        """
        self.performSelectorOnMainThread_withObject_waitUntilDone_modes_(
            b"_pyobjc_performOnThread:", (aSelector, arg), wait, modes
        )

    # And some a some versions that return results

    @objc.namedSelector(b"pyobjc_performSelectorOnMainThread:withObject:modes:")
    def pyobjc_performSelectorOnMainThread_withObject_modes_(
        self, aSelector, arg, modes
    ):
        """
        Simular to performSelectorOnMainThread:withObject:waitUntilDone:modes:,
        but:

        - always waits until done
        - returns the return value of the called method
        - if the called method raises an exception, this will raise the same
           exception
        """
        result = []
        self.performSelectorOnMainThread_withObject_waitUntilDone_modes_(
            b"_pyobjc_performOnThreadWithResult:", (aSelector, arg, result), True, modes
        )
        isOK, result = result[0]

        if isOK:
            return result
        else:
            exc_type, exc_value, exc_trace = result
            _raise(exc_type, exc_value, exc_trace)

    @objc.namedSelector(b"pyobjc_performSelectorOnMainThread:withObject:")
    def pyobjc_performSelectorOnMainThread_withObject_(self, aSelector, arg):
        result = []
        self.performSelectorOnMainThread_withObject_waitUntilDone_(
            b"_pyobjc_performOnThreadWithResult:", (aSelector, arg, result), True
        )
        isOK, result = result[0]

        if isOK:
            return result
        else:
            exc_type, exc_value, exc_trace = result
            _raise(exc_type, exc_value, exc_trace)

    if hasattr(NSObject, "performSelector_onThread_withObject_waitUntilDone_"):
        # These methods require Leopard, don't define them if the
        # platform functionality isn't present.

        @objc.namedSelector(b"pyobjc_performSelector:onThread:withObject:modes:")
        def pyobjc_performSelector_onThread_withObject_modes_(
            self, aSelector, thread, arg, modes
        ):
            result = []
            self.performSelector_onThread_withObject_waitUntilDone_modes_(
                b"_pyobjc_performOnThreadWithResult:",
                thread,
                (aSelector, arg, result),
                True,
                modes,
            )
            isOK, result = result[0]

            if isOK:
                return result
            else:
                exc_type, exc_value, exc_trace = result
                _raise(exc_type, exc_value, exc_trace)

        @objc.namedSelector(b"pyobjc_performSelector:onThread:withObject:")
        def pyobjc_performSelector_onThread_withObject_(self, aSelector, thread, arg):
            result = []
            self.performSelector_onThread_withObject_waitUntilDone_(
                b"_pyobjc_performOnThreadWithResult:",
                thread,
                (aSelector, arg, result),
                True,
            )
            isOK, result = result[0]

            if isOK:
                return result
            else:
                exc_type, exc_value, exc_trace = result
                _raise(exc_type, exc_value, exc_trace)


del NSObject



"""
Python mapping for the AppKit framework.

This module does not contain docstrings for the wrapped code, check Apple's
documentation for details on how to use these functions and classes.
"""
import sys

# Manually written wrappers:
import Foundation
import objc
from AppKit import _metadata
from AppKit._inlines import _inline_list_


def _setup_conveniences():
    def fontdescriptor_get(self, key, default=None):
        value = self.objectForKey_(key)
        if value is None:
            return default
        return value

    def fontdescriptor_getitem(self, key, default=None):
        value = self.objectForKey_(key)
        if value is None:
            raise KeyError(key)
        return value

    objc.addConvenienceForClass(
        "NSFontDescriptor",
        (("__getitem__", fontdescriptor_getitem), ("get", fontdescriptor_get)),
    )


_setup_conveniences()


def NSDictionaryOfVariableBindings(*names):
    """
    Return a dictionary with the given names and there values.
    """
    import sys

    variables = sys._getframe(1).f_locals

    return {nm: variables[nm] for nm in names}


sys.modules["AppKit"] = mod = objc.ObjCLazyModule(
    "AppKit",
    "com.apple.AppKit",
    objc.pathForFramework("/System/Library/Frameworks/AppKit.framework"),
    _metadata.__dict__,
    _inline_list_,
    {
        "__doc__": __doc__,
        "objc": objc,
        "NSDictionaryOfVariableBindings": NSDictionaryOfVariableBindings,
        "__path__": __path__,
        "__loader__": globals().get("__loader__", None),
    },
    (Foundation,),
)

# NSApp is a global variable that can be changed in ObjC,
# somewhat emulate that (it is *not* possible to assign to
# NSApp in Python)
from AppKit._nsapp import NSApp  # isort:skip  # noqa: E402

mod.NSApp = NSApp

import AppKit._AppKit  # isort:skip  # noqa: E402

for nm in dir(AppKit._AppKit):
    setattr(mod, nm, getattr(AppKit._AppKit, nm))

# Fix types for a number of character constants
mod.NSEnterCharacter = chr(mod.NSEnterCharacter)
mod.NSBackspaceCharacter = chr(mod.NSBackspaceCharacter)
mod.NSTabCharacter = chr(mod.NSTabCharacter)
mod.NSNewlineCharacter = chr(mod.NSNewlineCharacter)
mod.NSFormFeedCharacter = chr(mod.NSFormFeedCharacter)
mod.NSCarriageReturnCharacter = chr(mod.NSCarriageReturnCharacter)
mod.NSBackTabCharacter = chr(mod.NSBackTabCharacter)
mod.NSDeleteCharacter = chr(mod.NSDeleteCharacter)
mod.NSLineSeparatorCharacter = chr(mod.NSLineSeparatorCharacter)
mod.NSParagraphSeparatorCharacter = chr(mod.NSParagraphSeparatorCharacter)


for nm in [
    "NSUpArrowFunctionKey",
    "NSDownArrowFunctionKey",
    "NSLeftArrowFunctionKey",
    "NSRightArrowFunctionKey",
    "NSF1FunctionKey",
    "NSF2FunctionKey",
    "NSF3FunctionKey",
    "NSF4FunctionKey",
    "NSF5FunctionKey",
    "NSF6FunctionKey",
    "NSF7FunctionKey",
    "NSF8FunctionKey",
    "NSF9FunctionKey",
    "NSF10FunctionKey",
    "NSF11FunctionKey",
    "NSF12FunctionKey",
    "NSF13FunctionKey",
    "NSF14FunctionKey",
    "NSF15FunctionKey",
    "NSF16FunctionKey",
    "NSF17FunctionKey",
    "NSF18FunctionKey",
    "NSF19FunctionKey",
    "NSF20FunctionKey",
    "NSF21FunctionKey",
    "NSF22FunctionKey",
    "NSF23FunctionKey",
    "NSF24FunctionKey",
    "NSF25FunctionKey",
    "NSF26FunctionKey",
    "NSF27FunctionKey",
    "NSF28FunctionKey",
    "NSF29FunctionKey",
    "NSF30FunctionKey",
    "NSF31FunctionKey",
    "NSF32FunctionKey",
    "NSF33FunctionKey",
    "NSF34FunctionKey",
    "NSF35FunctionKey",
    "NSInsertFunctionKey",
    "NSDeleteFunctionKey",
    "NSHomeFunctionKey",
    "NSBeginFunctionKey",
    "NSEndFunctionKey",
    "NSPageUpFunctionKey",
    "NSPageDownFunctionKey",
    "NSPrintScreenFunctionKey",
    "NSScrollLockFunctionKey",
    "NSPauseFunctionKey",
    "NSSysReqFunctionKey",
    "NSBreakFunctionKey",
    "NSResetFunctionKey",
    "NSStopFunctionKey",
    "NSMenuFunctionKey",
    "NSUserFunctionKey",
    "NSSystemFunctionKey",
    "NSPrintFunctionKey",
    "NSClearLineFunctionKey",
    "NSClearDisplayFunctionKey",
    "NSInsertLineFunctionKey",
    "NSDeleteLineFunctionKey",
    "NSInsertCharFunctionKey",
    "NSDeleteCharFunctionKey",
    "NSPrevFunctionKey",
    "NSNextFunctionKey",
    "NSSelectFunctionKey",
    "NSExecuteFunctionKey",
    "NSUndoFunctionKey",
    "NSRedoFunctionKey",
    "NSFindFunctionKey",
    "NSHelpFunctionKey",
    "NSModeSwitchFunctionKey",
]:
    try:
        setattr(mod, nm, chr(getattr(mod, nm)))
    except AttributeError:
        pass

try:
    mod.NSImageNameApplicationIcon
except AttributeError:
    mod.NSImageNameApplicationIcon = "NSApplicationIcon"

if objc.arch == "arm64":
    # XXX: Temporary adjustment until the metadata
    #      is updated
    mod.NSImageResizingModeStretch = 1
    mod.NSImageResizingModeTile = 0

    mod.NSTextAlignmentCenter = 1
    mod.NSTextAlignmentRight = 2

mod.NSRightTextAlignment = mod.NSTextAlignmentRight
mod.NSCenterTextAlignment = mod.NSTextAlignmentCenter


del sys.modules["AppKit._metadata"]



"""
Python mapping for the CoreFoundation framework.

This module does not contain docstrings for the wrapped code, check Apple's
documentation for details on how to use these functions and classes.
"""
import sys

import objc
from CoreFoundation import _metadata
from CoreFoundation._inlines import _inline_list_

sys.modules["CoreFoundation"] = mod = objc.ObjCLazyModule(
    "CoreFoundation",
    "com.apple.CoreFoundation",
    objc.pathForFramework("/System/Library/Frameworks/CoreFoundation.framework"),
    _metadata.__dict__,
    _inline_list_,
    {
        "__doc__": __doc__,
        "__path__": __path__,
        "__loader__": globals().get("__loader__", None),
    },
    (),
)


import CoreFoundation._CoreFoundation  # isort:skip  # noqa: E402

for nm in dir(CoreFoundation._CoreFoundation):
    setattr(mod, nm, getattr(CoreFoundation._CoreFoundation, nm))

import CoreFoundation._static  # isort:skip  # noqa: E402

for nm in dir(CoreFoundation._static):
    setattr(mod, nm, getattr(CoreFoundation._static, nm))


del sys.modules["CoreFoundation._metadata"]



"""
Port of "function defines".
"""
from Foundation import NSBundle


def NSLocalizedString(key, comment):
    return NSBundle.mainBundle().localizedStringForKey_value_table_(key, "", None)


def NSLocalizedStringFromTable(key, tbl, comment):
    return NSBundle.mainBundle().localizedStringForKey_value_table_(key, "", tbl)


def NSLocalizedStringFromTableInBundle(key, tbl, bundle, comment):
    return bundle.localizedStringForKey_value_table_(key, "", tbl)


def NSLocalizedStringWithDefaultValue(key, tbl, bundle, val, comment):
    return bundle.localizedStringForKey_value_table_(key, val, tbl)


def NSLocalizedAttributedString(key, comment):
    return NSBundle.mainBundle().localizedAttributedStringForKey_value_table_(
        key, "", None
    )


def NSLocalizedAttributedStringFromTable(key, tbl, comment):
    return NSBundle.mainBundle.localizedAttributedStringForKey_value_table_(
        key, "", tbl
    )


def NSLocalizedAttributedStringFromTableInBundle(key, tbl, bundle, comment):
    return bundle.localizedAttributedStringForKey_value_table_(key, "", tbl)


def NSLocalizedAttributedStringWithDefaultValue(key, tbl, bundle, val, comment):
    return bundle.localizedAttributedStringForKey_value_table_(key, val, tbl)


def MIN(a, b):
    if a < b:
        return a
    else:
        return b


def MAX(a, b):
    if a < b:
        return b
    else:
        return a


ABS = abs
